Package: Gviz
Title: Plotting data and annotation information along genomic
        coordinates
Version: 1.57.1
Authors@R: c(person("Florian", "Hahne", role="aut"),
	 person("Steffen", "Durinck", role="aut"),
	 person("Robert", "Ivanek", role=c("aut", "cre"), 
	        email="robert.ivanek@unibas.ch", 
	        comment=c(ORCID="0000-0002-8403-056X")),
	 person("Arne", "Mueller", role="aut"),
	 person("Steve", "Lianoglou", role="aut"),
	 person("Ge", "Tan", role="aut"),
	 person("Lance", "Parsons", role="aut"),
	 person("Shraddha", "Pai", role="aut"),
	 person("Thomas", "McCarthy", role="ctb"),
	 person("Felix", "Ernst", role="ctb"),
	 person("Mike", "Smith", role="ctb"))
Description: Genomic data analyses requires integrated visualization of
        known genomic information and new experimental data. Gviz uses
        the biomaRt and the rtracklayer packages to perform live
        annotation queries to Ensembl and UCSC and translates this to
        e.g. gene/transcript structures in viewports of the grid
        graphics package. This results in genomic information plotted
        together with your data.
LazyLoad: yes
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Depends: R (>= 4.3), methods, S4Vectors (>= 0.9.25), IRanges (>=
        1.99.18), GenomicRanges (>= 1.61.1), grid
Imports: XVector (>= 0.5.7), rtracklayer (>= 1.69.1), lattice,
        RColorBrewer, biomaRt (>= 2.11.0), AnnotationDbi (>= 1.27.5),
        Biobase (>= 2.15.3), GenomicFeatures (>= 1.61.4), ensembldb (>=
        2.11.3), BSgenome (>= 1.77.1), Biostrings (>= 2.77.2),
        biovizBase (>= 1.13.8), Rsamtools (>= 2.25.1), latticeExtra (>=
        0.6-26), matrixStats (>= 0.8.14), GenomicAlignments (>=
        1.45.1), cigarillo, Seqinfo, GenomeInfoDb, BiocGenerics (>=
        0.11.3), digest(>= 0.6.8), graphics, grDevices, stats, utils
Suggests: BSgenome.Hsapiens.UCSC.hg19, xml2, BiocStyle, knitr,
        rmarkdown, testthat
biocViews: Visualization, Microarray, Sequencing
URL: https://github.com/ivanek/Gviz
BugReports: https://github.com/ivanek/Gviz/issues
VignetteBuilder: knitr
License: Artistic-2.0
Collate: 'utils.R' 'ImageMap-class.R' 'DisplayPars-class.R'
        'GdObject-class.R' 'ReferenceTrack-class.R'
        'SequenceTrack-class.R' 'RangeTrack-class.R'
        'StackedTrack-class.R' 'NumericTrack-class.R'
        'DataTrack-class.R' 'AlignmentsTrack-class.R'
        'AnnotationTrack-class.R' 'GeneRegionTrack-class.R'
        'BiomartGeneRegionTrack-class.R' 'CustomTrack-class.R'
        'GenomeAxisTrack-class.R' 'Gviz-defunct.R' 'Gviz-deprecated.R'
        'HighlightTrack-class.R' 'IdeogramTrack-class.R'
        'OverlayTrack-class.R' 'UcscTrack.R' 'collapsing.R'
        'datasets.R' 'exportTracks.R' 'grouping.R' 'plotTracks.R'
        'settings.R'
Config/roxygen2/version: 8.0.0
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-07-29 16:11:37 UTC
RemoteUrl: https://github.com/bioc/Gviz
RemoteRef: HEAD
RemoteSha: 2132e60faee98057a4ff849b6738b44e104060ea
NeedsCompilation: no
Packaged: 2026-07-29 21:44:35 UTC; root
Author: Florian Hahne [aut],
  Steffen Durinck [aut],
  Robert Ivanek [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8403-056X>),
  Arne Mueller [aut],
  Steve Lianoglou [aut],
  Ge Tan [aut],
  Lance Parsons [aut],
  Shraddha Pai [aut],
  Thomas McCarthy [ctb],
  Felix Ernst [ctb],
  Mike Smith [ctb]
Maintainer: Robert Ivanek <robert.ivanek@unibas.ch>
Built: R 4.6.1; ; 2026-07-29 21:48:10 UTC; unix
