# Generated by roxygen2: do not edit by hand

export(.DisplayPars)
export(.chrName)
export(.doCache)
export(AlignmentsTrack)
export(AnnotationTrack)
export(BiomartGeneRegionTrack)
export(CustomTrack)
export(DataTrack)
export(DetailsAnnotationTrack)
export(DisplayPars)
export(GeneRegionTrack)
export(GenomeAxisTrack)
export(HighlightTrack)
export(IdeogramTrack)
export(OverlayTrack)
export(RNASequenceTrack)
export(SequenceTrack)
export(UcscTrack)
export(addScheme)
export(availableDefaultMapping)
export(availableDisplayPars)
export(clearSessionCache)
export(drawAxis)
export(drawGD)
export(exportTracks)
export(getScheme)
export(plotTracks)
exportClasses(AlignmentsTrack)
exportClasses(AnnotationTrack)
exportClasses(BiomartGeneRegionTrack)
exportClasses(CustomTrack)
exportClasses(DataTrack)
exportClasses(DetailsAnnotationTrack)
exportClasses(DisplayPars)
exportClasses(GdObject)
exportClasses(GeneRegionTrack)
exportClasses(GenomeAxisTrack)
exportClasses(HighlightTrack)
exportClasses(IdeogramTrack)
exportClasses(ImageMap)
exportClasses(NumericTrack)
exportClasses(OverlayTrack)
exportClasses(RangeTrack)
exportClasses(ReferenceAlignmentsTrack)
exportClasses(ReferenceAnnotationTrack)
exportClasses(ReferenceDataTrack)
exportClasses(ReferenceGeneRegionTrack)
exportClasses(ReferenceSequenceTrack)
exportClasses(ReferenceTrack)
exportClasses(SequenceBSgenomeTrack)
exportClasses(SequenceDNAStringSetTrack)
exportClasses(SequenceRNAStringSetTrack)
exportClasses(SequenceTrack)
exportClasses(StackedTrack)
exportMethods("[")
exportMethods("chromosome<-")
exportMethods("displayPars<-")
exportMethods("end<-")
exportMethods("exon<-")
exportMethods("feature<-")
exportMethods("gene<-")
exportMethods("genome<-")
exportMethods("group<-")
exportMethods("identifier<-")
exportMethods("imageMap<-")
exportMethods("stacking<-")
exportMethods("start<-")
exportMethods("strand<-")
exportMethods("symbol<-")
exportMethods("transcript<-")
exportMethods("values<-")
exportMethods("width<-")
exportMethods(as.list)
exportMethods(chromosome)
exportMethods(consolidateTrack)
exportMethods(coords)
exportMethods(displayPars)
exportMethods(drawAxis)
exportMethods(drawGD)
exportMethods(end)
exportMethods(exon)
exportMethods(feature)
exportMethods(gene)
exportMethods(genome)
exportMethods(getPar)
exportMethods(group)
exportMethods(identifier)
exportMethods(imageMap)
exportMethods(initialize)
exportMethods(length)
exportMethods(max)
exportMethods(min)
exportMethods(position)
exportMethods(range)
exportMethods(seqinfo)
exportMethods(seqlevels)
exportMethods(seqnames)
exportMethods(setCoverage)
exportMethods(setPar)
exportMethods(setStacks)
exportMethods(show)
exportMethods(split)
exportMethods(stacking)
exportMethods(stacks)
exportMethods(start)
exportMethods(strand)
exportMethods(subset)
exportMethods(symbol)
exportMethods(tags)
exportMethods(transcript)
exportMethods(values)
exportMethods(width)
import(BiocGenerics)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(Seqinfo)
import(XVector)
import(grid)
import(methods)
importClassesFrom(BSgenome,BSgenome)
importClassesFrom(BSgenome,MaskedBSgenome)
importClassesFrom(Biostrings,BString)
importClassesFrom(Biostrings,BStringSet)
importClassesFrom(Biostrings,DNAString)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(Biostrings,RNAString)
importClassesFrom(Biostrings,RNAStringSet)
importClassesFrom(GenomicFeatures,TxDb)
importClassesFrom(biomaRt,Mart)
importClassesFrom(ensembldb,EnsDb)
importClassesFrom(rtracklayer,UCSCData)
importFrom(BSgenome,bsapply)
importFrom(Biobase,listLen)
importFrom(Biobase,rowMax)
importFrom(Biobase,rowMin)
importFrom(Biostrings,BString)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,DNA_ALPHABET)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,stackStrings)
importFrom(GenomicAlignments,GAlignments)
importFrom(GenomicAlignments,extractAlignmentRangesOnReference)
importFrom(GenomicAlignments,summarizeJunctions)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rsamtools,BamFile)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,bamWhich)
importFrom(Rsamtools,scanBam)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,scanBamHeader)
importFrom(Rsamtools,scanBamWhat)
importFrom(Rsamtools,scanFa)
importFrom(Rsamtools,scanFaIndex)
importFrom(biomaRt,getBM)
importFrom(biomaRt,listAttributes)
importFrom(biomaRt,listDatasets)
importFrom(biomaRt,listFilters)
importFrom(biomaRt,useEnsembl)
importFrom(biomaRt,useMart)
importFrom(biovizBase,getBioColor)
importFrom(cigarillo,cigars_as_ranges_along_ref)
importFrom(cigarillo,project_sequences)
importFrom(digest,digest)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,par)
importFrom(lattice,current.panel.limits)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.grid)
importFrom(lattice,panel.lines)
importFrom(lattice,panel.points)
importFrom(lattice,panel.polygon)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.text)
importFrom(lattice,panel.xyplot)
importFrom(lattice,trellis.par.get)
importFrom(latticeExtra,panel.horizonplot)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowMins)
importFrom(rtracklayer,BigWigFile)
importFrom(rtracklayer,FastaFile)
importFrom(rtracklayer,GenomicData)
importFrom(rtracklayer,TwoBitFile)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,ucscGenomes)
importFrom(stats,loess.smooth)
importFrom(stats,setNames)
importFrom(utils,assignInNamespace)
importFrom(utils,modifyList)
importFrom(utils,type.convert)
importFrom(utils,write.table)
importMethodsFrom(AnnotationDbi,colnames)
importMethodsFrom(AnnotationDbi,get)
importMethodsFrom(AnnotationDbi,ls)
importMethodsFrom(AnnotationDbi,mget)
importMethodsFrom(AnnotationDbi,tail)
importMethodsFrom(BSgenome,providerVersion)
importMethodsFrom(BSgenome,seqnames)
importMethodsFrom(Biostrings,"seqtype<-")
importMethodsFrom(Biostrings,complement)
importMethodsFrom(Biostrings,consensusMatrix)
importMethodsFrom(Biostrings,consensusString)
importMethodsFrom(Biostrings,seqtype)
importMethodsFrom(Biostrings,unmasked)
importMethodsFrom(GenomicFeatures,"isActiveSeq<-")
importMethodsFrom(GenomicFeatures,cdsBy)
importMethodsFrom(GenomicFeatures,exonsBy)
importMethodsFrom(GenomicFeatures,fiveUTRsByTranscript)
importMethodsFrom(GenomicFeatures,isActiveSeq)
importMethodsFrom(GenomicFeatures,threeUTRsByTranscript)
importMethodsFrom(GenomicFeatures,transcripts)
importMethodsFrom(GenomicFeatures,transcriptsBy)
importMethodsFrom(ensembldb,cdsBy)
importMethodsFrom(ensembldb,exonsBy)
importMethodsFrom(ensembldb,fiveUTRsByTranscript)
importMethodsFrom(ensembldb,threeUTRsByTranscript)
importMethodsFrom(ensembldb,transcripts)
importMethodsFrom(ensembldb,transcriptsBy)
importMethodsFrom(rtracklayer,"tableName<-")
importMethodsFrom(rtracklayer,chrom)
importMethodsFrom(rtracklayer,close)
importMethodsFrom(rtracklayer,getTable)
importMethodsFrom(rtracklayer,import)
importMethodsFrom(rtracklayer,import.2bit)
importMethodsFrom(rtracklayer,import.bed)
importMethodsFrom(rtracklayer,import.bed15)
importMethodsFrom(rtracklayer,import.bedGraph)
importMethodsFrom(rtracklayer,import.bw)
importMethodsFrom(rtracklayer,import.chain)
importMethodsFrom(rtracklayer,import.gff)
importMethodsFrom(rtracklayer,import.gff1)
importMethodsFrom(rtracklayer,import.gff2)
importMethodsFrom(rtracklayer,import.gff3)
importMethodsFrom(rtracklayer,import.ucsc)
importMethodsFrom(rtracklayer,import.wig)
importMethodsFrom(rtracklayer,seqinfo)
importMethodsFrom(rtracklayer,tableNames)
importMethodsFrom(rtracklayer,track)
importMethodsFrom(rtracklayer,trackNames)
importMethodsFrom(rtracklayer,ucscTableQuery)
