#' Data sets
#'
#' Some sample data sets used for the illustrative examples and the vignette.
#'
#' @name datasets
#'
#' @aliases bmTrack cyp2b10 idTrack biomTrack biomTrack2 cpgIslands axTrack
#' conservation ensGenes denseAnnTrack geneModels iTrack itrack idxTrack
#' ideoTrack twoGroups from gcContent idTrack knownGenes refGenes snpLocations
#' to ctrack geneDetails dtHoriz bmt
#'
#' @docType data
#' @keywords datasets
NULL
